/* demo_tray.c

	gcc -o demo_tray demo_tray.c -w `pkg-config --libs gtk+-3.0` `pkg-config --cflags gtk+-3.0` 
	gcc -o demo_tray demo_tray.c -w `pkg-config --libs gtk+-2.0` `pkg-config --cflags gtk+-2.0`
	
  	A tray icon demo app for linux to monitor the existence of a file.
  	Green means yes, red means no.
  	
  	To test from a separate console window:
  	root# touch /tmp/traydemo.txt
	root# rm /tmp/traydemo.txt
 
  	- jafadmin - November 2020
*/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gstdio.h>

////// 	Images resources
#include "green-check.xpm"
#include "red-cross.xpm"
GdkPixbuf *green_check_pixbuf;
GdkPixbuf *red_cross_pixbuf;
////// 	End images ...

#define FILE_EXISTS	0
#define CONTINUE	1

char test_file_name[]="/tmp/traydemo.txt";

GtkStatusIcon *tray_icon;
unsigned int interval = 3000; /* Update interval in milliseconds ( 1 second = 1000 milliseconds) */

gboolean ChkStatus(gpointer ptr) // State switcher. This does all the work ..
{
	if(access(test_file_name, F_OK) == FILE_EXISTS)  // Does file exist?
	{
		gtk_status_icon_set_from_pixbuf(tray_icon, green_check_pixbuf); 	
		gtk_status_icon_set_tooltip_text(tray_icon, "File is there");
	}		
	else
	{ 								
		gtk_status_icon_set_from_pixbuf(tray_icon, red_cross_pixbuf); 	
		gtk_status_icon_set_tooltip_text(tray_icon, "File is missing");	
	}	
    return CONTINUE;        			
}

////// The following two functions handle left & right clicks on the tray icon
void tray_icon_on_left_click(GtkStatusIcon *status_icon,  gpointer user_data) 
{
	char DeleteCmd[80];	// Make a shell command to delete the testfile
	
	strcpy(DeleteCmd, "rm ");	
	strcat(DeleteCmd, test_file_name);
    system(DeleteCmd);    		
}

void tray_icon_on_right_click(GtkStatusIcon *status_icon, guint button,  guint activate_time, gpointer user_data)
{
	char TouchCmd[80];	// Recreate the file using 'touch'
	
	strcpy(TouchCmd, "touch ");	
	strcat(TouchCmd, test_file_name); 
    system(TouchCmd);    
	
	// Or .. you could just have it:
    // gtk_main_quit();
}
////// End clicks  ...

static GtkStatusIcon *create_tray_icon() 
{
    GtkStatusIcon *tray_icon;
    tray_icon = gtk_status_icon_new();
    
    green_check_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)green_check_xpm);
    red_cross_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)red_cross_xpm);

	// Connect handler functions to signals from X
    g_signal_connect(G_OBJECT(tray_icon), "activate", G_CALLBACK(tray_icon_on_left_click), NULL);
    g_signal_connect(G_OBJECT(tray_icon), "popup-menu", G_CALLBACK(tray_icon_on_right_click), NULL);
    
    gtk_status_icon_set_tooltip_text(tray_icon,  "Demo tray app");
    gtk_status_icon_set_visible(tray_icon, TRUE);
    
    return tray_icon;
}

int main(int argc, char **argv) 
{		
	gtk_init(&argc, &argv);        
    tray_icon = create_tray_icon();        
    g_timeout_add(interval, ChkStatus, NULL);	// Call our worker function, ChkStatus()
    gtk_main();
    return 0;
}

